@@ -1,5 +1,3 @@
-use preserves_schema::Codec;
-
 use std::io;
 use std::path::PathBuf;
 use std::sync::Arc;
@@ -8,62 +6,62 @@ use syndicate::actor::*;
 use syndicate::enclose;
 use syndicate::error::Error;
 use syndicate::relay;
+use syndicate::supervise::{Supervisor, SupervisorConfiguration};
 
 use tokio::net::UnixListener;
 use tokio::net::UnixStream;
 
 use crate::language::language;
+use crate::lifecycle;
 use crate::protocol::run_connection;
-use crate::schemas::internal_services;
+use crate::schemas::internal_services::UnixRelayListener;
 
 use syndicate_macros::during;
 
-pub fn on_demand(t: &mut Activation, ds: Arc<Cap>, gateway: Arc<Cap>) {
-    t.spawn(syndicate::name!("on_demand", module = module_path!()), move |t| {
-        Ok(during!(t, ds, language(), <run-service $spec: internal_services::UnixRelayListener>,
-                   |t: &mut Activation| {
-                       t.spawn_link(syndicate::name!(parent: None, "relay", addr = ?spec),
-                                    enclose!((ds, gateway) |t| run(t, ds, gateway, spec)));
-                       Ok(())
-                   }))
+pub fn on_demand(t: &mut Activation, ds: Arc<Cap>) {
+    t.spawn(syndicate::name!("unix_relay_listener"), move |t| {
+        Ok(during!(t, ds, language(), <run-service $spec: UnixRelayListener>, |t| {
+            Supervisor::start(
+                t,
+                syndicate::name!(parent: None, "relay", addr = ?spec),
+                SupervisorConfiguration::default(),
+                enclose!((ds, spec) lifecycle::updater(ds, spec)),
+                enclose!((ds) move |t| enclose!((ds, spec) run(t, ds, spec))))
+        }))
     });
 }
 
-fn run(
-    t: &'_ mut Activation,
-    ds: Arc<Cap>,
-    gateway: Arc<Cap>,
-    spec: internal_services::UnixRelayListener,
-) -> ActorResult {
+fn run(t: &mut Activation, ds: Arc<Cap>, spec: UnixRelayListener) -> ActorResult {
     let path_str = spec.addr.path.clone();
-    {
-        let spec = language().unparse(&spec);
-        ds.assert(t, &(), &syndicate_macros::template!("<service-running =spec>"));
-    }
     let parent_span = tracing::Span::current();
+    let facet = t.facet.clone();
     t.linked_task(syndicate::name!("listener"), async move {
         let listener = bind_unix_listener(&PathBuf::from(path_str)).await?;
-        tracing::info!("listening");
+        facet.activate(Account::new(syndicate::name!("readiness")), |t| {
+            tracing::info!("listening");
+            ds.assert(t, language(), &lifecycle::ready(&spec));
+            Ok(())
+        })?;
         loop {
             let (stream, _addr) = listener.accept().await?;
             let peer = stream.peer_cred()?;
+            let gatekeeper = spec.gatekeeper.clone();
             Actor::new().boot(
                 syndicate::name!(parent: parent_span.clone(), "conn",
                                  pid = ?peer.pid().unwrap_or(-1),
                                  uid = peer.uid()),
-                enclose!((gateway) |t| Ok(t.linked_task(
-                    tracing::Span::current(),
-                    {
-                        let facet = t.facet.clone();
-                        async move {
-                            tracing::info!(protocol = %"unix");
-                            let (i, o) = stream.into_split();
-                            run_connection(facet,
-                                           relay::Input::Bytes(Box::pin(i)),
-                                           relay::Output::Bytes(Box::pin(o)),
-                                           gateway)
-                        }
-                    }))));
+                |t| Ok(t.linked_task(tracing::Span::current(), {
+                    let facet = t.facet.clone();
+                    async move {
+                        tracing::info!(protocol = %"unix");
+                        let (i, o) = stream.into_split();
+                        run_connection(facet,
+                                       relay::Input::Bytes(Box::pin(i)),
+                                       relay::Output::Bytes(Box::pin(o)),
+                                       gatekeeper)?;
+                        Ok(LinkedTaskTermination::KeepFacet)
+                    }
+                })));
         }
     });
     Ok(())
@@ -80,7 +78,7 @@ async fn bind_unix_listener(path: &PathBuf) -> Result<UnixListener, Error> {
                 Ok(_probe) => Err(e)?, // Someone's already there! Give up.
                 Err(f) if f.kind() == io::ErrorKind::ConnectionRefused => {
                     // Try to steal the socket.
-                    tracing::info!("Cleaning stale socket");
+                    tracing::debug!("Cleaning stale socket");
                     std::fs::remove_file(path)?;
                     Ok(UnixListener::bind(path)?)
                 }
